@@ -1,16 +1,19 @@
 from __future__ import print_function
+
 import sys
 from os import environ
 from base64 import b64encode
 import json
 from datetime import datetime, timedelta
+from itertools import chain
 
 import requests
+from requests import HTTPError
 from urlobject import URLObject
 import six
 from six.moves.urllib.parse import urlencode
 from nylas._client_sdk_version import __VERSION__
-from nylas.client.errors import MessageRejectedError
+from nylas.client.errors import MessageRejectedError, NylasApiError
 from nylas.client.restful_model_collection import RestfulModelCollection
 from nylas.client.restful_models import (
     Calendar,
@@ -26,8 +29,12 @@ from nylas.client.restful_models import (
     Folder,
     Label,
     Draft,
+    Component,
 )
 from nylas.client.neural_api_models import Neural
+from nylas.client.scheduler_restful_model_collection import (
+    SchedulerRestfulModelCollection,
+)
 from nylas.utils import timestamp_from_dt, create_request_body
 
 DEBUG = environ.get("NYLAS_CLIENT_DEBUG")
@@ -55,8 +62,9 @@ def _validate(response):
         # we will handle it separate and handle a _different_ exception
         # so that users don't think they need to pay.
         raise MessageRejectedError(response)
+    elif response.status_code >= 400:
+        raise NylasApiError(response)
 
-    response.raise_for_status()
     return response
 
 
@@ -249,8 +257,35 @@ class APIClient(json.JSONEncoder):
         _validate(resp)
         return resp.json()
 
+    def open_hours(self, emails, days, timezone, start, end):
+        if isinstance(emails, six.string_types):
+            emails = [emails]
+        if isinstance(days, int):
+            days = [days]
+        if isinstance(start, datetime):
+            start = "{hour}:{minute}".format(hour=start.hour, minute=start.minute)
+        if isinstance(start, datetime):
+            end = "{hour}:{minute}".format(hour=end.hour, minute=end.minute)
+        return {
+            "emails": emails,
+            "days": days,
+            "timezone": timezone,
+            "start": start,
+            "end": end,
+            "object_type": "open_hours",
+        }
+
     def availability(
-        self, emails, duration, interval, start_at, end_at, free_busy=None
+        self,
+        emails,
+        duration,
+        interval,
+        start_at,
+        end_at,
+        buffer=None,
+        round_robin=None,
+        free_busy=None,
+        open_hours=None,
     ):
         if isinstance(emails, six.string_types):
             emails = [emails]
@@ -270,6 +305,9 @@ class APIClient(json.JSONEncoder):
             end_time = timestamp_from_dt(end_at)
         else:
             end_time = end_at
+        if open_hours is not None:
+            self._validate_open_hours(emails, open_hours, free_busy)
+
         url = "{api_server}/calendars/availability".format(api_server=self.api_server)
         data = {
             "emails": emails,
@@ -277,7 +315,61 @@ class APIClient(json.JSONEncoder):
             "interval_minutes": interval_minutes,
             "start_time": start_time,
             "end_time": end_time,
+            "buffer": buffer,
+            "round_robin": round_robin,
             "free_busy": free_busy or [],
+            "open_hours": open_hours or [],
+        }
+        resp = self.session.post(url, json=data)
+        _validate(resp)
+        return resp.json()
+
+    def consecutive_availability(
+        self,
+        emails,
+        duration,
+        interval,
+        start_at,
+        end_at,
+        buffer=None,
+        free_busy=None,
+        open_hours=None,
+    ):
+        if isinstance(emails, six.string_types):
+            emails = [[emails]]
+        elif isinstance(emails[0], list) is False:
+            raise ValueError("'emails' must be a list of lists.")
+        if isinstance(duration, timedelta):
+            duration_minutes = int(duration.total_seconds() // 60)
+        else:
+            duration_minutes = int(duration)
+        if isinstance(interval, timedelta):
+            interval_minutes = int(interval.total_seconds() // 60)
+        else:
+            interval_minutes = int(interval)
+        if isinstance(start_at, datetime):
+            start_time = timestamp_from_dt(start_at)
+        else:
+            start_time = start_at
+        if isinstance(end_at, datetime):
+            end_time = timestamp_from_dt(end_at)
+        else:
+            end_time = end_at
+        if open_hours is not None:
+            self._validate_open_hours(emails, open_hours, free_busy)
+
+        url = "{api_server}/calendars/availability/consecutive".format(
+            api_server=self.api_server
+        )
+        data = {
+            "emails": emails,
+            "duration_minutes": duration_minutes,
+            "interval_minutes": interval_minutes,
+            "start_time": start_time,
+            "end_time": end_time,
+            "buffer": buffer,
+            "free_busy": free_busy or [],
+            "open_hours": open_hours or [],
         }
         resp = self.session.post(url, json=data)
         _validate(resp)
@@ -333,6 +425,14 @@ class APIClient(json.JSONEncoder):
     def calendars(self):
         return RestfulModelCollection(Calendar, self)
 
+    @property
+    def scheduler(self):
+        return SchedulerRestfulModelCollection(self)
+
+    @property
+    def components(self):
+        return RestfulModelCollection(Component, self)
+
     @property
     def neural(self):
         return Neural(self)
@@ -345,7 +445,7 @@ class APIClient(json.JSONEncoder):
         # Is this a request for a resource under the accounts/billing/admin
         # namespace (/a)? If the latter, pass the client_secret
         # instead of the secret_token
-        if api_root == "a":
+        if api_root:
             return self.admin_session
         return self.session
 
@@ -353,11 +453,18 @@ class APIClient(json.JSONEncoder):
         # FIXME @karim: remove this interim code when we've got rid
         # of the old accounts API.
         postfix = "/{}".format(extra) if extra else ""
-        if cls.api_root != "a":
-            url = "{}/{}{}".format(self.api_server, cls.collection_name, postfix)
+        path = "/{}".format(cls.collection_name) if cls.collection_name else ""
+        if not cls.api_root:
+            url = "{server}{path}{postfix}".format(
+                server=self.api_server, path=path, postfix=postfix
+            )
         else:
-            url = "{}/a/{}/{}{}".format(
-                self.api_server, self.client_id, cls.collection_name, postfix
+            url = "{server}/{prefix}/{client_id}{path}{postfix}".format(
+                server=self.api_server,
+                prefix=cls.api_root,
+                client_id=self.client_id,
+                path=path,
+                postfix=postfix,
             )
 
         converted_data = create_request_body(filters, cls.datetime_filter_attrs)
@@ -367,25 +474,36 @@ class APIClient(json.JSONEncoder):
         return [cls.create(self, **x) for x in results if x is not None]
 
     def _get_resource_raw(
-        self, cls, id, extra=None, headers=None, stream=False, **filters
+        self, cls, id, extra=None, headers=None, stream=False, path=None, **filters
     ):
         """Get an individual REST resource"""
-        headers = headers or {}
-        headers.update(self.session.headers)
-
+        if path is None:
+            path = cls.collection_name
         postfix = "/{}".format(extra) if extra else ""
-        if cls.api_root != "a":
-            url = "{}/{}/{}{}".format(self.api_server, cls.collection_name, id, postfix)
+        path = "/{}".format(path) if path else ""
+        id = "/{}".format(id) if id else ""
+        if not cls.api_root:
+            url = "{server}{path}{id}{postfix}".format(
+                server=self.api_server, path=path, id=id, postfix=postfix
+            )
         else:
-            url = "{}/a/{}/{}/{}{}".format(
-                self.api_server, self.client_id, cls.collection_name, id, postfix
+            url = "{server}/{prefix}/{client_id}{path}{id}{postfix}".format(
+                server=self.api_server,
+                prefix=cls.api_root,
+                client_id=self.client_id,
+                path=path,
+                id=id,
+                postfix=postfix,
             )
 
         converted_data = create_request_body(filters, cls.datetime_filter_attrs)
         url = str(URLObject(url).add_query_params(converted_data.items()))
-        response = self._get_http_session(cls.api_root).get(
-            url, headers=headers, stream=stream
-        )
+
+        session = self._get_http_session(cls.api_root)
+
+        headers = headers or {}
+        headers.update(session.headers)
+        response = session.get(url, headers=headers, stream=stream)
         return _validate(response)
 
     def _get_resource(self, cls, id, **filters):
@@ -402,9 +520,15 @@ class APIClient(json.JSONEncoder):
         return response.content
 
     def _create_resource(self, cls, data, **kwargs):
+        name = "{prefix}{path}".format(
+            prefix="/{}/{}".format(cls.api_root, self.client_id)
+            if cls.api_root
+            else "",
+            path="/{}".format(cls.collection_name) if cls.collection_name else "",
+        )
         url = (
             URLObject(self.api_server)
-            .with_path("/{name}/".format(name=cls.collection_name))
+            .with_path("{name}".format(name=name))
             .set_query_params(**kwargs)
         )
 
@@ -415,7 +539,7 @@ class APIClient(json.JSONEncoder):
         else:
             converted_data = create_request_body(data, cls.datetime_attrs)
             headers = {"Content-Type": "application/json"}
-            headers.update(self.session.headers)
+            headers.update(session.headers)
             response = session.post(url, json=converted_data, headers=headers)
 
         result = _validate(response).json()
@@ -424,9 +548,13 @@ class APIClient(json.JSONEncoder):
         return cls.create(self, **result)
 
     def _create_resources(self, cls, data):
-        url = URLObject(self.api_server).with_path(
-            "/{name}/".format(name=cls.collection_name)
+        name = "{prefix}{path}".format(
+            prefix="/{}/{}".format(cls.api_root, self.client_id)
+            if cls.api_root
+            else "",
+            path="/{}".format(cls.collection_name) if cls.collection_name else "",
         )
+        url = URLObject(self.api_server).with_path("{name}".format(name=name))
         session = self._get_http_session(cls.api_root)
 
         if cls == File:
@@ -436,16 +564,22 @@ class APIClient(json.JSONEncoder):
                 create_request_body(datum, cls.datetime_attrs) for datum in data
             ]
             headers = {"Content-Type": "application/json"}
-            headers.update(self.session.headers)
+            headers.update(session.headers)
             response = session.post(url, json=converted_data, headers=headers)
 
         results = _validate(response).json()
         return [cls.create(self, **x) for x in results]
 
     def _delete_resource(self, cls, id, data=None, **kwargs):
+        name = "{prefix}{path}".format(
+            prefix="/{}/{}".format(cls.api_root, self.client_id)
+            if cls.api_root
+            else "",
+            path="/{}".format(cls.collection_name) if cls.collection_name else "",
+        )
         url = (
             URLObject(self.api_server)
-            .with_path("/{name}/{id}".format(name=cls.collection_name, id=id))
+            .with_path("{name}/{id}".format(name=name, id=id))
             .set_query_params(**kwargs)
         )
         session = self._get_http_session(cls.api_root)
@@ -454,10 +588,20 @@ class APIClient(json.JSONEncoder):
         else:
             _validate(session.delete(url))
 
-    def _update_resource(self, cls, id, data, **kwargs):
+    def _put_resource(self, cls, id, data, extra=None, path=None, **kwargs):
+        if path is None:
+            path = cls.collection_name
+        name = "{prefix}{path}".format(
+            prefix="/{}/{}".format(cls.api_root, self.client_id)
+            if cls.api_root
+            else "",
+            path="/{}".format(path) if path else "",
+        )
+
+        postfix = "/{}".format(extra) if extra else ""
         url = (
             URLObject(self.api_server)
-            .with_path("/{name}/{id}".format(name=cls.collection_name, id=id))
+            .with_path("{name}/{id}{postfix}".format(name=name, id=id, postfix=postfix))
             .set_query_params(**kwargs)
         )
 
@@ -466,22 +610,27 @@ class APIClient(json.JSONEncoder):
         converted_data = create_request_body(data, cls.datetime_attrs)
         response = session.put(url, json=converted_data)
 
-        result = _validate(response).json()
+        result = _validate(response)
+        return result.json()
+
+    def _update_resource(self, cls, id, data, **kwargs):
+        result = self._put_resource(cls, id, data, kwargs)
         return cls.create(self, **result)
 
-    def _call_resource_method(self, cls, id, method_name, data):
-        """POST a dictionary to an API method,
-        for example /a/.../accounts/id/upgrade"""
-
-        if cls.api_root != "a":
-            url_path = "/{name}/{id}/{method}".format(
-                name=cls.collection_name, id=id, method=method_name
+    def _post_resource(self, cls, id, method_name, data, path=None):
+        if path is None:
+            path = cls.collection_name
+        path = "/{}".format(path) if path else ""
+        if not cls.api_root:
+            url_path = "{name}/{id}/{method}".format(
+                name=path, id=id, method=method_name
             )
         else:
             # Management method.
-            url_path = "/a/{client_id}/{name}/{id}/{method}".format(
+            url_path = "/{prefix}/{client_id}{path}/{id}/{method}".format(
+                prefix=cls.api_root,
                 client_id=self.client_id,
-                name=cls.collection_name,
+                path=path,
                 id=id,
                 method=method_name,
             )
@@ -492,7 +641,13 @@ class APIClient(json.JSONEncoder):
         session = self._get_http_session(cls.api_root)
         response = session.post(url, json=converted_data)
 
-        result = _validate(response).json()
+        return _validate(response).json()
+
+    def _call_resource_method(self, cls, id, method_name, data):
+        """POST a dictionary to an API method,
+        for example /a/.../accounts/id/upgrade"""
+
+        result = self._post_resource(cls, id, method_name, data)
         return cls.create(self, **result)
 
     def _request_neural_resource(self, cls, data, path=None, method="PUT"):
@@ -513,3 +668,20 @@ class APIClient(json.JSONEncoder):
             return object_list
 
         return cls.create(self, **result)
+
+    def _validate_open_hours(self, emails, open_hours, free_busy):
+        if isinstance(open_hours, list) is False:
+            raise ValueError("'open_hours' must be an array.")
+        open_hours_emails = list(
+            chain.from_iterable([oh["emails"] for oh in open_hours])
+        )
+        free_busy_emails = (
+            [fb["email"] for fb in free_busy] if free_busy is not None else []
+        )
+        if isinstance(emails[0], list) is True:
+            emails = list(chain.from_iterable(emails))
+        for email in open_hours_emails:
+            if (email in emails) is False and (email in free_busy_emails) is False:
+                raise ValueError(
+                    "Open Hours cannot contain an email not present in the main email list or the free busy email list."
+                )
