@@ -110,7 +110,7 @@ def mocked_responses():
 
 @pytest.fixture
 def mock_save_draft(mocked_responses, api_url):
-    save_endpoint = re.compile(api_url + "/drafts/")
+    save_endpoint = re.compile(api_url + "/drafts")
     response_body = json.dumps(
         {"id": "4dl0ni6vxomazo73r5oydo16k", "version": "4dw0ni6txomazo33r5ozdo16j"}
     )
@@ -173,6 +173,13 @@ def mock_accounts(mocked_responses, api_url, account_id, client_id):
             return (200, {}, json.dumps([]))
         return (200, {}, json.dumps(accounts))
 
+    def update_callback(request):
+        response = accounts[0]
+        payload = json.loads(request.body)
+        if payload["metadata"]:
+            response["metadata"] = payload["metadata"]
+        return 200, {}, json.dumps(response)
+
     url_re = "{base}(/a/{client_id})?/accounts/?".format(
         base=api_url, client_id=client_id
     )
@@ -182,6 +189,12 @@ def mock_accounts(mocked_responses, api_url, account_id, client_id):
         content_type="application/json",
         callback=list_callback,
     )
+    mocked_responses.add_callback(
+        responses.PUT,
+        re.compile(url_re),
+        content_type="application/json",
+        callback=update_callback,
+    )
 
 
 @pytest.fixture
@@ -391,6 +404,8 @@ def mock_message(mocked_responses, api_url, account_id):
                 for l in payload["labels"]
             ]
             base_msg["labels"] = labels
+        if "metadata" in payload:
+            base_msg["metadata"] = payload["metadata"]
         return (200, {}, json.dumps(base_msg))
 
     endpoint = re.compile(api_url + "/messages/1234")
@@ -641,7 +656,7 @@ def mock_draft_saved_response(mocked_responses, api_url):
 
     mocked_responses.add_callback(
         responses.POST,
-        api_url + "/drafts/",
+        api_url + "/drafts",
         content_type="application/json",
         callback=create_callback,
     )
@@ -699,7 +714,7 @@ def mock_draft_sent_response(mocked_responses, api_url):
 
     mocked_responses.add_callback(
         responses.POST,
-        api_url + "/send/",
+        api_url + "/send",
         callback=callback,
         content_type="application/json",
     )
@@ -714,7 +729,7 @@ def mock_draft_send_unsaved_response(mocked_responses, api_url):
 
     mocked_responses.add_callback(
         responses.POST,
-        api_url + "/send/",
+        api_url + "/send",
         callback=callback,
         content_type="application/json",
     )
@@ -771,7 +786,7 @@ def mock_files(mocked_responses, api_url, account_id):
         return (200, {}, json.dumps(body))
 
     mocked_responses.add_callback(
-        responses.POST, api_url + "/files/", callback=create_callback
+        responses.POST, api_url + "/files", callback=create_callback
     )
 
 
@@ -787,7 +802,7 @@ def mock_event_create_response(mocked_responses, api_url, message_body):
         return 200, {}, json.dumps(payload)
 
     mocked_responses.add_callback(
-        responses.POST, api_url + "/events/", callback=callback
+        responses.POST, api_url + "/events", callback=callback
     )
 
     put_body = {"title": "loaded from JSON", "ignored": "ignored"}
@@ -798,6 +813,28 @@ def mock_event_create_response(mocked_responses, api_url, message_body):
     )
 
 
+@pytest.fixture
+def mock_scheduler_create_response(mocked_responses, api_url, message_body):
+    def callback(_request):
+        try:
+            payload = json.loads(_request.body)
+        except ValueError:
+            return 400, {}, ""
+
+        payload["id"] = "cv4ei7syx10uvsxbs21ccsezf"
+        return 200, {}, json.dumps(payload)
+
+    mocked_responses.add_callback(
+        responses.POST, "https://api.schedule.nylas.com/manage/pages", callback=callback
+    )
+
+    mocked_responses.add(
+        responses.PUT,
+        "https://api.schedule.nylas.com/manage/pages/cv4ei7syx10uvsxbs21ccsezf",
+        body=json.dumps(message_body),
+    )
+
+
 @pytest.fixture
 def mock_event_create_response_with_limits(mocked_responses, api_url, message_body):
     def callback(request):
@@ -813,7 +850,7 @@ def mock_event_create_response_with_limits(mocked_responses, api_url, message_bo
 def mock_event_create_notify_response(mocked_responses, api_url, message_body):
     mocked_responses.add(
         responses.POST,
-        api_url + "/events/?notify_participants=true&other_param=1",
+        api_url + "/events?notify_participants=true&other_param=1",
         body=json.dumps(message_body),
     )
 
@@ -827,6 +864,28 @@ def mock_send_rsvp(mocked_responses, api_url, message_body):
     )
 
 
+@pytest.fixture
+def mock_components_create_response(mocked_responses, api_url, message_body):
+    def callback(_request):
+        try:
+            payload = json.loads(_request.body)
+        except ValueError:
+            return 400, {}, ""
+
+        payload["id"] = "cv4ei7syx10uvsxbs21ccsezf"
+        return 200, {}, json.dumps(payload)
+
+    mocked_responses.add_callback(
+        responses.POST, re.compile(api_url + "/component/*"), callback=callback
+    )
+
+    mocked_responses.add(
+        responses.PUT,
+        re.compile(api_url + "/component/*"),
+        body=json.dumps(message_body),
+    )
+
+
 @pytest.fixture
 def mock_thread_search_response(mocked_responses, api_url):
     snippet = (
@@ -1094,7 +1153,7 @@ def mock_contacts(mocked_responses, account_id, api_url):
     )
     mocked_responses.add_callback(
         responses.POST,
-        api_url + "/contacts/",
+        api_url + "/contacts",
         content_type="application/json",
         callback=create_callback,
     )
@@ -1268,6 +1327,276 @@ def mock_events(mocked_responses, api_url):
     )
 
 
+@pytest.fixture
+def mock_schedulers(mocked_responses, api_url):
+    scheduler_list = [
+        {
+            "app_client_id": "test-client-id",
+            "app_organization_id": 12345,
+            "config": {
+                "appearance": {
+                    "color": "#0068D3",
+                    "company_name": "",
+                    "logo": "",
+                    "show_autoschedule": "true",
+                    "show_nylas_branding": "false",
+                    "show_timezone_options": "true",
+                    "show_week_view": "true",
+                    "submit_text": "Submit",
+                },
+                "locale": "en",
+                "reminders": [],
+                "timezone": "America/Los_Angeles",
+            },
+            "created_at": "2021-10-22",
+            "edit_token": "test-edit-token-1",
+            "id": 90210,
+            "modified_at": "2021-10-22",
+            "name": "test-1",
+            "slug": "test1",
+        },
+        {
+            "app_client_id": "test-client-id",
+            "app_organization_id": 12345,
+            "config": {
+                "calendar_ids": {
+                    "test-calendar-id": {
+                        "availability": ["availability-id"],
+                        "booking": "booking-id",
+                    }
+                },
+                "event": {
+                    "capacity": -1,
+                    "duration": 45,
+                    "location": "Location TBD",
+                    "title": "test-event",
+                },
+                "locale": "en",
+                "reminders": [],
+                "timezone": "America/Los_Angeles",
+            },
+            "created_at": "2021-10-22",
+            "edit_token": "test-edit-token-2",
+            "id": 90211,
+            "modified_at": "2021-10-22",
+            "name": "test-2",
+            "slug": "test2",
+        },
+    ]
+
+    def list_callback(arg=None):
+        return 200, {}, json.dumps(scheduler_list)
+
+    def return_one_callback(arg=None):
+        return 200, {}, json.dumps(scheduler_list[0])
+
+    info_endpoint = re.compile("https://api.schedule.nylas.com/schedule/.*/info")
+
+    mocked_responses.add_callback(
+        responses.GET,
+        "https://api.schedule.nylas.com/manage/pages",
+        content_type="application/json",
+        callback=list_callback,
+    )
+
+    mocked_responses.add_callback(
+        responses.GET,
+        info_endpoint,
+        content_type="application/json",
+        callback=return_one_callback,
+    )
+
+
+@pytest.fixture
+def mock_scheduler_get_available_calendars(mocked_responses, api_url):
+    calendars = [
+        {
+            "calendars": [
+                {"id": "calendar-id", "name": "Emailed events", "read_only": "true"},
+            ],
+            "email": "swag@nylas.com",
+            "id": "scheduler-id",
+            "name": "Python Tester",
+        }
+    ]
+
+    def list_callback(arg=None):
+        return 200, {}, json.dumps(calendars)
+
+    calendars_url = "https://api.schedule.nylas.com/manage/pages/{id}/calendars".format(
+        id="cv4ei7syx10uvsxbs21ccsezf"
+    )
+
+    mocked_responses.add_callback(
+        responses.GET,
+        calendars_url,
+        content_type="application/json",
+        callback=list_callback,
+    )
+
+
+@pytest.fixture
+def mock_scheduler_upload_image(mocked_responses, api_url):
+    upload = {
+        "filename": "test.png",
+        "originalFilename": "test.png",
+        "publicUrl": "https://public.nylas.com/test.png",
+        "signedUrl": "https://signed.nylas.com/test.png",
+    }
+
+    def list_callback(arg=None):
+        return 200, {}, json.dumps(upload)
+
+    calendars_url = (
+        "https://api.schedule.nylas.com/manage/pages/{id}/upload-image".format(
+            id="cv4ei7syx10uvsxbs21ccsezf"
+        )
+    )
+
+    mocked_responses.add_callback(
+        responses.PUT,
+        calendars_url,
+        content_type="application/json",
+        callback=list_callback,
+    )
+
+
+@pytest.fixture
+def mock_scheduler_provider_availability(mocked_responses, api_url):
+    response = {
+        "busy": [
+            {
+                "end": 1636731958,
+                "start": 1636728347,
+            },
+        ],
+        "email": "test@example.com",
+        "name": "John Doe",
+    }
+
+    def callback(arg=None):
+        return 200, {}, json.dumps(response)
+
+    provider_url = re.compile(
+        "https://api.schedule.nylas.com/schedule/availability/(google|o365)"
+    )
+
+    mocked_responses.add_callback(
+        responses.GET,
+        provider_url,
+        callback=callback,
+    )
+
+
+@pytest.fixture
+def mock_scheduler_timeslots(mocked_responses, api_url):
+    scheduler_time_slots = [
+        {
+            "account_id": "test-account-id",
+            "calendar_id": "test-calendar-id",
+            "emails": ["test@example.com"],
+            "end": 1636731958,
+            "host_name": "www.hostname.com",
+            "start": 1636728347,
+        },
+    ]
+
+    booking_confirmation = {
+        "account_id": "test-account-id",
+        "additional_field_values": {
+            "test": "yes",
+        },
+        "calendar_event_id": "test-event-id",
+        "calendar_id": "test-calendar-id",
+        "edit_hash": "test-edit-hash",
+        "end_time": 1636731958,
+        "id": 123,
+        "is_confirmed": False,
+        "location": "Earth",
+        "recipient_email": "recipient@example.com",
+        "recipient_locale": "en_US",
+        "recipient_name": "Recipient Doe",
+        "recipient_tz": "America/New_York",
+        "start_time": 1636728347,
+        "title": "Test Booking",
+    }
+
+    cancel_payload = {
+        "success": True,
+    }
+
+    def list_timeslots(arg=None):
+        return 200, {}, json.dumps(scheduler_time_slots)
+
+    def book_timeslot(arg=None):
+        return 200, {}, json.dumps(booking_confirmation)
+
+    def confirm_booking(arg=None):
+        booking_confirmation["is_confirmed"] = True
+        return 200, {}, json.dumps(booking_confirmation)
+
+    def cancel_booking(arg=None):
+        return 200, {}, json.dumps(cancel_payload)
+
+    timeslots_url = re.compile("https://api.schedule.nylas.com/schedule/.*/timeslots")
+
+    confirm_url = re.compile("https://api.schedule.nylas.com/schedule/.*/.*/confirm")
+
+    cancel_url = re.compile("https://api.schedule.nylas.com/schedule/.*/.*/cancel")
+
+    mocked_responses.add_callback(
+        responses.GET,
+        timeslots_url,
+        callback=list_timeslots,
+    )
+
+    mocked_responses.add_callback(
+        responses.POST,
+        timeslots_url,
+        callback=book_timeslot,
+    )
+
+    mocked_responses.add_callback(
+        responses.POST,
+        confirm_url,
+        callback=confirm_booking,
+    )
+
+    mocked_responses.add_callback(
+        responses.POST,
+        cancel_url,
+        callback=cancel_booking,
+    )
+
+
+@pytest.fixture
+def mock_components(mocked_responses, api_url):
+    components = [
+        {
+            "active": True,
+            "settings": {},
+            "allowed_domains": [],
+            "id": "component-id",
+            "name": "PyTest Component",
+            "public_account_id": "account-id",
+            "public_application_id": "application-id",
+            "type": "agenda",
+            "created_at": "2021-10-22T18:02:10.000Z",
+            "updated_at": "2021-10-22T18:02:10.000Z",
+            "accessed_at": None,
+            "public_token_id": "token-id",
+        },
+    ]
+
+    def list_callback(arg=None):
+        return 200, {}, json.dumps(components)
+
+    endpoint = re.compile(api_url + "/component/*")
+    mocked_responses.add_callback(
+        responses.GET, endpoint, content_type="application/json", callback=list_callback
+    )
+
+
 @pytest.fixture
 def mock_resources(mocked_responses, api_url):
     resources = [
@@ -1468,6 +1797,13 @@ def mock_availability(mocked_responses, api_url):
         callback=availability_callback,
     )
 
+    mocked_responses.add_callback(
+        responses.POST,
+        "{url}/consecutive".format(url=availability_url),
+        content_type="application/json",
+        callback=availability_callback,
+    )
+
 
 @pytest.fixture
 def mock_sentiment_analysis(mocked_responses, api_url, account_id):
