@@ -39,7 +39,7 @@ def test_event_notify(mocked_responses, api_client):
 
 
 @pytest.mark.usefixtures("mock_event_create_response")
-def test_event_conferencing(mocked_responses, api_client):
+def test_event_conferencing_details(mocked_responses, api_client):
     event = blank_event(api_client)
     event.conferencing = {
         "provider": "Zoom Meeting",
@@ -72,6 +72,49 @@ def test_event_conferencing(mocked_responses, api_client):
     assert body["conferencing"]["details"]["phone"] == ["+11234567890"]
 
 
+@pytest.mark.usefixtures("mock_event_create_response")
+def test_event_conferencing_autocreate(mocked_responses, api_client):
+    event = blank_event(api_client)
+    event.conferencing = {
+        "provider": "Zoom Meeting",
+        "autocreate": {
+            "settings": {},
+        },
+    }
+    event.save()
+    assert event.id == "cv4ei7syx10uvsxbs21ccsezf"
+    assert event.conferencing["provider"] == "Zoom Meeting"
+    assert event.conferencing["autocreate"]["settings"] == {}
+
+    body = json.loads(mocked_responses.calls[-1].request.body)
+    assert body["conferencing"]["provider"] == "Zoom Meeting"
+    assert event.conferencing["autocreate"]["settings"] == {}
+
+
+@pytest.mark.usefixtures("mock_event_create_response")
+def test_event_conferencing_details_autocreate_error(mocked_responses, api_client):
+    event = blank_event(api_client)
+    event.conferencing = {
+        "provider": "Zoom Meeting",
+        "details": {
+            "url": "https://us02web.zoom.us/j/****************",
+            "meeting_code": "213",
+            "password": "xyz",
+            "phone": ["+11234567890"],
+        },
+        "autocreate": {
+            "settings": {
+                "password": "1234",
+            },
+        },
+    }
+    with pytest.raises(ValueError) as excinfo:
+        event.save()
+    assert "Cannot set both 'details' and 'autocreate' in conferencing object." in str(
+        excinfo
+    )
+
+
 @pytest.mark.usefixtures("mock_calendars", "mock_events")
 def test_calendar_events(api_client):
     calendar = api_client.calendars.first()
@@ -281,6 +324,7 @@ def test_availability_with_free_busy(mocked_responses, api_client):
                     "end_time": 1584379800,
                 }
             ],
+            "object": "free_busy",
         }
     ]
     availability = api_client.availability(
@@ -303,6 +347,160 @@ def test_availability_with_free_busy(mocked_responses, api_client):
     assert data["free_busy"] == free_busy
 
 
+@pytest.mark.usefixtures("mock_availability")
+def test_consecutive_availability(mocked_responses, api_client):
+    emails = [["one@example.com"], ["two@example.com", "three@example.com"]]
+    duration = timedelta(minutes=30)
+    interval = timedelta(hours=1, minutes=30)
+    start_at = datetime(2020, 1, 1)
+    end_at = datetime(2020, 1, 2)
+    open_hours = api_client.open_hours(
+        ["one@example.com", "two@example.com", "three@example.com"],
+        [0],
+        "America/Chicago",
+        "10:00",
+        "14:00",
+    )
+    api_client.consecutive_availability(
+        emails, duration, interval, start_at, end_at, open_hours=[open_hours]
+    )
+
+    request = mocked_responses.calls[-1].request
+    assert URLObject(request.url).path == "/calendars/availability/consecutive"
+    data = json.loads(request.body)
+    assert data["emails"] == emails
+    assert data["duration_minutes"] == 30
+    assert isinstance(data["duration_minutes"], int)
+    assert data["interval_minutes"] == 90
+    assert isinstance(data["interval_minutes"], int)
+    assert data["start_time"] == 1577836800
+    assert data["end_time"] == 1577923200
+    assert data["free_busy"] == []
+    assert data["open_hours"][0]["emails"] == [
+        "one@example.com",
+        "two@example.com",
+        "three@example.com",
+    ]
+    assert data["open_hours"][0]["days"] == [0]
+    assert data["open_hours"][0]["timezone"] == "America/Chicago"
+    assert data["open_hours"][0]["start"] == "10:00"
+    assert data["open_hours"][0]["end"] == "14:00"
+
+
+@pytest.mark.usefixtures("mock_availability")
+def test_consecutive_availability_free_busy(mocked_responses, api_client):
+    emails = [["one@example.com"], ["two@example.com", "three@example.com"]]
+    duration = timedelta(minutes=30)
+    interval = timedelta(hours=1, minutes=30)
+    start_at = datetime(2020, 1, 1)
+    end_at = datetime(2020, 1, 2)
+    open_hours = api_client.open_hours(
+        [
+            "one@example.com",
+            "two@example.com",
+            "three@example.com",
+            "visitor@external.net",
+        ],
+        [0],
+        "America/Chicago",
+        "10:00",
+        "14:00",
+    )
+    free_busy = [
+        {
+            "email": "visitor@external.net",
+            "time_slots": [
+                {
+                    "object": "time_slot",
+                    "status": "busy",
+                    "start_time": 1584377898,
+                    "end_time": 1584379800,
+                }
+            ],
+            "object": "free_busy",
+        }
+    ]
+    api_client.consecutive_availability(
+        emails,
+        duration,
+        interval,
+        start_at,
+        end_at,
+        free_busy=free_busy,
+        open_hours=[open_hours],
+    )
+
+    request = mocked_responses.calls[-1].request
+    assert URLObject(request.url).path == "/calendars/availability/consecutive"
+    data = json.loads(request.body)
+    assert data["emails"] == emails
+    assert data["duration_minutes"] == 30
+    assert isinstance(data["duration_minutes"], int)
+    assert data["interval_minutes"] == 90
+    assert isinstance(data["interval_minutes"], int)
+    assert data["start_time"] == 1577836800
+    assert data["end_time"] == 1577923200
+    assert data["free_busy"] == free_busy
+    assert data["open_hours"][0]["emails"] == [
+        "one@example.com",
+        "two@example.com",
+        "three@example.com",
+        "visitor@external.net",
+    ]
+    assert data["open_hours"][0]["days"] == [0]
+    assert data["open_hours"][0]["timezone"] == "America/Chicago"
+    assert data["open_hours"][0]["start"] == "10:00"
+    assert data["open_hours"][0]["end"] == "14:00"
+
+
+@pytest.mark.usefixtures("mock_availability")
+def test_consecutive_availability_invalid_open_hours_email(
+    mocked_responses, api_client
+):
+    emails = [["one@example.com"], ["two@example.com", "three@example.com"]]
+    duration = timedelta(minutes=30)
+    interval = timedelta(hours=1, minutes=30)
+    start_at = datetime(2020, 1, 1)
+    end_at = datetime(2020, 1, 2)
+    open_hours = api_client.open_hours(
+        [
+            "one@example.com",
+            "two@example.com",
+            "three@example.com",
+            "visitor@external.net",
+            "four@example.com",
+        ],
+        [0],
+        "America/Chicago",
+        "10:00",
+        "14:00",
+    )
+    free_busy = [
+        {
+            "email": "visitor@external.net",
+            "time_slots": [
+                {
+                    "object": "time_slot",
+                    "status": "busy",
+                    "start_time": 1584377898,
+                    "end_time": 1584379800,
+                }
+            ],
+            "object": "free_busy",
+        }
+    ]
+    with pytest.raises(ValueError):
+        api_client.consecutive_availability(
+            emails,
+            duration,
+            interval,
+            start_at,
+            end_at,
+            free_busy=free_busy,
+            open_hours=[open_hours],
+        )
+
+
 @pytest.mark.usefixtures("mock_events")
 def test_metadata_filtering(api_client):
     events_filtered_by_key = api_client.events.where(metadata_key="platform")
@@ -327,3 +525,23 @@ def test_metadata_filtering(api_client):
 
     non_existant_event = api_client.events.where(metadata_pair={"bla": "blablabla"})
     assert len(non_existant_event.all()) == 0
+
+
+@pytest.mark.usefixtures("mock_event_create_response")
+def test_event_notifications(mocked_responses, api_client):
+    event = blank_event(api_client)
+    event.notifications = [
+        {
+            "type": "email",
+            "minutes_before_event": 60,
+            "subject": "Test Event Notification",
+            "body": "Reminding you about our meeting.",
+        }
+    ]
+    event.save()
+    assert event.id == "cv4ei7syx10uvsxbs21ccsezf"
+    assert len(event.notifications) == 1
+    assert event.notifications[0]["type"] == "email"
+    assert event.notifications[0]["minutes_before_event"] == 60
+    assert event.notifications[0]["subject"] == "Test Event Notification"
+    assert event.notifications[0]["body"] == "Reminding you about our meeting."
